%%  Introduction
%{
    Requires one to run qtMSBFEM.m first in order to obtain the relevant stiffness matrices
    change "MS.nCNodes" in the script qtMSBFEM to appropriate value
%}
    clear RVE
%%  Parameters
    RVE.length = 513;
    RVE.nWidth = 6;
    RVE.nHeight = 5;
    RVE.wallWidth  = RVE.nWidth*(RVE.length-1)+1;
    RVE.wallHeight = RVE.nHeight*(RVE.length-1)+1;
    
%%  Initialize

%%  Precompute
    %connecting layer
    ls1_nodes = (MS.nCNodes/4)*RVE.nWidth+1;
    ls1 = linspace(1,RVE.wallWidth,ls1_nodes);
    %internal layer
    ls2_nodes = RVE.nWidth + 1;
    ls2 = linspace(1,RVE.wallWidth,ls2_nodes);
    %vertical layer all heights
    ls3_nodes = (MS.nCNodes/4)*RVE.nHeight+1;
    ls3 = linspace(1,RVE.wallHeight,ls3_nodes);
    %vertical layer connecting heights
    ls4_nodes = RVE.nHeight + 1;
    ls4 = linspace(1,RVE.wallHeight,ls4_nodes);
    
    %find positions of connecting layers withing whole height
    index = ismembertol(ls3,ls4);
    
%%  Create node scatter
    RVE.points = [];
    for i = 1:1:ls3_nodes
        %check if connecting of internal layer
        if index(i)
            %connecting layer
            RVE.points = [RVE.points;[ls1',ls3(i).*ones(length(ls1),1)]];
        else
            %boundary layer
            RVE.points = [RVE.points;[ls2',ls3(i).*ones(length(ls2),1)]];
        end
    end
    
%%  calculate each RVE's center point
    xStart = (1+RVE.length)/2;
    xEnd   = ls1(end)-xStart+1;
    yStart = xStart;
    yEnd   = ls4(end)-yStart+1;
    
    lsX = linspace(xStart,xEnd,RVE.nWidth);
    lsY = linspace(yStart,yEnd,RVE.nHeight);
    
    [X,Y] = meshgrid(lsX,lsY);
    
    X = X';
    Y = Y';
    
    RVE.centers = [X(:),Y(:)];
    
%%  Determine nodes associated with each RVE by radius search
    RVE.nCElements = RVE.nWidth*RVE.nHeight;
    xP = RVE.points(:,1);
    yP = RVE.points(:,2);
    factor = 1.05;
    RVE.nodes = zeros(RVE.nCElements,MS.nCNodes);
    RVE.DOF = zeros(RVE.nCElements,MS.nCNodes*qt.DOFPerNode);
    for i = 1:1:RVE.nCElements
        sortedNodes = [];
        xC = RVE.centers(i,1);
        yC = RVE.centers(i,2);
        
        distance = sqrt((xP-xC).^2+(yP-yC).^2);
        [row,~,~] = find(distance<RVE.length/2*sqrt(2)*factor);
        nodeI = RVE.points(row,:);
        %correct for relative position
        nodes(:,1) = nodeI(:,1)-min(nodeI(:,1))+1;
        nodes(:,2) = nodeI(:,2)-min(nodeI(:,2))+1;
        
        %side 1
        [row,~,~] = find(nodes(:,1)<1*factor);
        nodesTemp = nodes(row,:);
        nodesTemp = sortrows(nodesTemp,2);
        sortedNodes = [sortedNodes;nodesTemp];
        %side 2
        [row,~,~] = find(nodes(:,2)>RVE.length/factor);
        nodesTemp = nodes(row,:);
        nodesTemp = sortrows(nodesTemp,1);
        sortedNodes = [sortedNodes;nodesTemp];
        %side 3
        [row,~,~] = find(nodes(:,1)>RVE.length/factor);
        nodesTemp = nodes(row,:);
        nodesTemp = flipud(sortrows(nodesTemp,2));
        sortedNodes = [sortedNodes;nodesTemp];
        %side 4
        [row,~,~] = find(nodes(:,2)<1*factor);
        nodesTemp = nodes(row,:);
        nodesTemp = flipud(sortrows(nodesTemp,2));
        sortedNodes = [sortedNodes;nodesTemp];
        %determine unique sorted nodes
        sortedNodes = unique(sortedNodes,'rows','stable');
        sortedNodes(:,1) = sortedNodes(:,1)+min(nodeI(:,1))-1;
        sortedNodes(:,2) = sortedNodes(:,2)+min(nodeI(:,2))-1;
        %find these rows in RVE,points
        [~,iB] = ismember(sortedNodes,RVE.points,'rows');
        
        RVE.nodes(i,:) = iB';
        
        %determine corresponding global DOF
        RVE.temp = repmat(RVE.nodes(i,:)',[1,2]);
        RVE.temp(:,1) = RVE.temp(:,1)*qt.DOFPerNode-1;
        RVE.temp(:,2) = RVE.temp(:,2)*qt.DOFPerNode;
        RVE.temp = RVE.temp';
        RVE.DOF(i,:) = reshape(RVE.temp,1,[]);
    end
        
%%  Assemble global coarse stiffness matrix
    RVE.nDOF = length(RVE.points) * qt.DOFPerNode;
    RVE.K = zeros(RVE.nDOF,RVE.nDOF);
    for i = 1:1:RVE.nCElements
        ind = RVE.DOF(i,:);
        RVE.K(ind,ind) = RVE.K(ind,ind) + qt.kCoarse;
    end

%%  Construct global coarse loading vector
    %nodes and DOF associated with y = 513 (top)
    [row,~,~] = find(RVE.points(:,2)>RVE.wallHeight /factor);
    load = -qt.load;
    RVE.F = zeros(RVE.nDOF,1);
    for  i=1:1:length(row)-1
        %determine force
        xS = RVE.points(row(i),1);
        yS = RVE.points(row(i),2);
        xE = RVE.points(row(i+1),1);
        yE = RVE.points(row(i+1),2);
        distance = sqrt((xS-xE)^2+(yS-yE)^2);
        force = distance * load * 0.5;
        
        %determine DOF
        rows = repmat(row(i:i+1),[1,2]);
        rows(:,1) = rows(:,1)*qt.DOFPerNode-1;
        rows(:,2) = rows(:,2)*qt.DOFPerNode;
        rows = rows';
        rows = reshape(rows,1,[]);
        eDOF = rows;
        
        %determine local force vector
        fLoc = [force,0,force,0]; %shear
        %fLoc = [0,force,0,force]; %tension
        
        %assemble
        RVE.F(eDOF,1) = RVE.F(eDOF,1)+fLoc';
    end

%%  Apply boundary conditions
    %find DOF associated with y = 0
        [row,~,~] = find(RVE.points(:,2)<1*factor);
        %construct DOF from these point indices
        RVE.BC = repmat(row,[1,2]);
        RVE.BC(:,1) = RVE.BC(:,1)*qt.DOFPerNode-1;
        RVE.BC(:,2) = RVE.BC(:,2)*qt.DOFPerNode;
        RVE.BC = RVE.BC';
        RVE.BC = reshape(RVE.BC,1,[]);
        
    %find DOF assocoated with y = top and y-direction
        [row,~,~] = find(RVE.points(:,2)>RVE.wallHeight/factor);
        %construct DOF from these point indices
        BC = repmat(row,[1,2]);
        BC(:,1) = BC(:,1)*qt.DOFPerNode-1;
        BC(:,2) = BC(:,2)*qt.DOFPerNode;
        BC = BC';
        BC = reshape(BC,1,[]);
        %take only the y-direction
        BC = BC(2:2:end);
        %add to bottom BCs
        RVE.BC = [RVE.BC,BC];
    
    %stiffness matrix
    RVE.K(RVE.BC,     :) = zeros;
    RVE.K(     :,RVE.BC) = zeros;
    RVE.K(RVE.BC,RVE.BC) = eye(length(RVE.BC));

%%  Solve for displacements at coarse nodes
    RVE.U = RVE.K\RVE.F;
%return
%%  Calculate averaged strains at each node of each RVE
    %create storage for stresses and strains
    for  i= 1:1:RVE.nCElements
        RVE.UC(i).strain(1:5,length(qt.gNodes)) = zeros;
        RVE.UC(i).stress(1:5,length(qt.gNodes)) = zeros;
    end

    for i = 1:1:RVE.nCElements
        [ qt, MS, blocks, RVE ] = calcRVEStrains( i, qt, MS, blocks, RVE );
    end
%return
%%  Interpolate the strains and stresses to meshgrid for one RVE
%{
    F = scatteredInterpolant;
    F.Points = [qt.gNodes(:,1),qt.gNodes(:,2)];
    F.Values = RVE.UC(1).strain(1,:)';
    F.Method = 'natural';
    
    [xI, yI] = meshgrid(1:1:513);
    
    vI = F(xI,yI);
   
    hold on; 
    pcolor(vI);
    colormap(parula(1000))
    shading interp;
    axis equal
    contour(xI,yI,vI,'LineColor','k');
    xlabel('X','fontweight','b'), ylabel('Y','fontweight','b');
    zlabel('Value - V','fontweight','b');
    title('Natural Interpolation Method','fontweight','b');
    hold off
%}

%%  Interpolate the strains and stress to meshgrid for all RVEs
%{
f1 = figure;
    for  i = 1:1:RVE.nCElements
        F = scatteredInterpolant;
        F.Points = [qt.gNodes(:,1),qt.gNodes(:,2)];
        F.Values = RVE.UC(i).stress(5,:)';
        F.Method = 'natural';

        [xI, yI] = meshgrid(1:1:513);
        
        vI = F(xI,yI);
        
        %shift xI, yI according to RVE position
        points = RVE.points(RVE.nodes(i,:),:);
        xMin = min(points(:,1));
        yMin = min(points(:,2));
        xI = xI + xMin-1;
        yI = yI + yMin-1;
        
        
        hold on; 
        pcolor(xI,yI,vI);
        shading interp;
        axis equal
        axis off
        %contour(xI,yI,vI,'LineColor','k');
        %xlabel('X','fontweight','b'), ylabel('Y','fontweight','b');
        %zlabel('Value - V','fontweight','b');
        %title('Natural Interpolation Method','fontweight','b');
        hold off
    end
        hold on
        colormap(parula(1000))
        x = RVE.points(:,1);
        y = RVE.points(:,2);
        scatter(x,y,'k')
        axis off
        axis equal
        hold off
%}        
%%  Interpolate the displacements to meshgrid for all RVEs
%%{
f2 = figure;
    for  i = 1:1:RVE.nCElements
        F = scatteredInterpolant;
        F.Points = [qt.gNodes(:,1),qt.gNodes(:,2)];
        
        %determine the coarse displacements of this RVE
        cDisp = RVE.U(RVE.DOF(i,:));
        %determine the fine displacements of this RVE
        uDisp = MS.NBF * cDisp;
        Mag = sqrt(uDisp(1:2:end).^2+uDisp(2:2:end).^2);
        F.Values = Mag;
        F.Method = 'natural';

        [xI, yI] = meshgrid(1:1:513);
        
        vI = F(xI,yI);
        
        %shift xI, yI according to RVE position
        points = RVE.points(RVE.nodes(i,:),:);
        xMin = min(points(:,1));
        yMin = min(points(:,2));
        xI = xI + xMin-1;
        yI = yI + yMin-1;
        
        
        hold on; 
        pcolor(xI,yI,vI);
        shading interp;
        axis equal
        axis off
        %contour(xI,yI,vI,'LineColor','k');
        %xlabel('X','fontweight','b'), ylabel('Y','fontweight','b');
        %zlabel('Value - V','fontweight','b');
        %title('Natural Interpolation Method','fontweight','b');
        hold off
    end
        hold on
        colormap(parula(1000))
        x = RVE.points(:,1);
        y = RVE.points(:,2);
        scatter(x,y,'k')
        axis off
        axis equal
        hold off
%}
%%  Plot displaced shape
%{
    %general parameters and values
    scaleFactor = 10;
    
    %multiscale SBFEM coarse
    x = RVE.points(:,1);
    y = RVE.points(:,2);
    
    uX_MS_c = RVE.U(1:2:end);
    uY_MS_c = RVE.U(2:2:end);
    
    dispX_MS_c = x + scaleFactor * uX_MS_c;
    dispY_MS_c = y + scaleFactor * uY_MS_c;
    
    %plot
    f3 = figure;
    hold on
    scatter(x,y)
    scatter(dispX_MS_c,dispY_MS_c)
    legend('nodes','multiscale coarse')
    axis off
    axis equal
    hold off
%}   
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
    
    
   