%%  Introduction
%{
    Requires one to run qtMSBFEM.m first in order to obtain the relevant stiffness matrices
    change "MS.nCNodes" in the script qtMSBFEM to appropriate value
%}
    clear SD
%%  Parameters
    SD.length = 513;
    SD.nWidth = 6;
    SD.nHeight = 5;
    SD.nSD     = SD.nWidth * SD.nHeight;
    SD.wallWidth  = SD.nWidth*(SD.length-1)+1;
    SD.wallHeight = SD.nHeight*(SD.length-1)+1;
    SD.tol = 10^-3;
    
%%  generate nodes for each SD
    SD.gNodes = [];
    for i = 1:1:SD.nSD
        yLayer = floor(i/(SD.nWidth+SD.tol));
        xRve   = i - yLayer * SD.nWidth;
        tempNodesX = qt.gNodes(:,1) + (SD.length - 1) * (xRve - 1);
        tempNodesY = qt.gNodes(:,2) + (SD.length - 1) * (yLayer);
        SD.gNodes = [SD.gNodes;[tempNodesX, tempNodesY]];
    end
    
%%  generate unique nodes and linking of qt-DOF to SD-DOF
    %starting and ending node of each SD
    SD.nodesIndex = zeros(SD.nSD,2);
    for i = 1:1:SD.nSD
        startNode = 1 + (i-1)*length(qt.gNodes);
        stopNode  = i * length(qt.gNodes);
        SD.nodesIndex(i,:) = [startNode,stopNode];
    end
    
    %unique sorting of all generated nodes
    [SD.gNodes,indL, indG] = unique(SD.gNodes,'rows','stable');

    %assign global SD-DOF indices to each qt-DOF
    SD.nodes = zeros(SD.nSD,length(qt.gNodes));
    for i = 1:1:SD.nSD
        SD.nodes(i,:) = indG(SD.nodesIndex(i,1):SD.nodesIndex(i,2),1);
    end

    %generate SD-DOF from SD.nodes
    SD.gDOF = zeros(SD.nSD,qt.DOFPerNode * length(qt.gNodes));
    for i = 1:1:SD.nSD
        DOF = repmat(SD.nodes(i,:)',[1,2]);
        DOF(:,1) = DOF(:,1)*qt.DOFPerNode-1;
        DOF(:,2) = DOF(:,2)*qt.DOFPerNode;
        DOF = DOF';
        DOF = reshape(DOF,1,[]);
        SD.gDOF(i,:) = reshape(DOF,1,[]);
    end
    
%%  Assemble full wall stiffness matrix
    SD.nDOF = length(SD.gNodes) * qt.DOFPerNode;
    SD.K = sparse(SD.nDOF,SD.nDOF);
    
    for i = 1:1:SD.nSD
        ind = SD.gDOF(i,:);
        SD.K(ind,ind) = SD.K(ind,ind) + qt.kFine;
    end

%%  Construct full Wall load vector
    %correct previous orientation error
        %find block numbers at top of domain
        qt.topBlocks = [];
        for i = 1:1:qt.nBlocks
            if find(blocks(i).nodes(:,2) > qt.imageSize - 2)
                qt.topBlocks = [qt.topBlocks,i];
            end
        end
        [F] = getForceVector(blocks, qt); 
        
    %only the top SD's are loaded in shear
    SD.F = zeros(SD.nDOF,1);
    for i = 25:1:SD.nSD
        ind = SD.gDOF(i,:);
        SD.F(ind,1) = SD.F(ind,1) + F;
    end

%%  Apply boundary conditions
    %find DOF associated with y = 0
        [row,~,~] = find(SD.gNodes(:,2) < 2);
        %construct DOF from these point indices
        SD.BC = repmat(row,[1,2]);
        SD.BC(:,1) = SD.BC(:,1)*qt.DOFPerNode-1;
        SD.BC(:,2) = SD.BC(:,2)*qt.DOFPerNode;
        SD.BC = SD.BC';
        SD.BC = reshape(SD.BC,1,[]);
        
    %find DOF assocoated with y = top and y-direction
        [row,~,~] = find(SD.gNodes(:,2) > SD.wallHeight - 2 );
        %construct DOF from these point indices
        BC = repmat(row,[1,2]);
        BC(:,1) = BC(:,1)*qt.DOFPerNode-1;
        BC(:,2) = BC(:,2)*qt.DOFPerNode;
        BC = BC';
        BC = reshape(BC,1,[]);
        %take only the y-direction
        BC = BC(2:2:end);
        %add to bottom BCs
        SD.BC = [SD.BC,BC];
    
    %stiffness matrix
    SD.K(SD.BC,     :) = zeros;
    SD.K(     :,SD.BC) = zeros;
    SD.K(SD.BC,SD.BC)  = eye(length(SD.BC));
    
    %force vector
    SD.F(SD.BC,:)      = zeros;

%%  Solve for displacements at coarse nodes
spparms('spumoni',2)
    SD.U = SD.K\SD.F;
return
%%  Calculate averaged strains at each node of each SD
    %create storage for stresses and strains
    for  i= 1:1:SD.nSD
        SD.UC(i).strain(1:5,length(qt.gNodes)) = zeros;
        SD.UC(i).stress(1:5,length(qt.gNodes)) = zeros;
    end

    for i = 1:1:SD.nSD
        [ qt, MS, blocks, SD ] = calcSDStrains( i, qt, MS, blocks, SD );
    end

%%  Interpolate the strains and stresses to meshgrid for one SD
%{
    F = scatteredInterpolant;
    F.Points = [qt.gNodes(:,1),qt.gNodes(:,2)];
    F.Values = SD.UC(1).strain(1,:)';
    F.Method = 'natural';
    
    [xI, yI] = meshgrid(1:1:513);
    
    vI = F(xI,yI);
   
    hold on; 
    pcolor(vI);
    colormap(parula(1000))
    shading interp;
    axis equal
    contour(xI,yI,vI,'LineColor','k');
    xlabel('X','fontweight','b'), ylabel('Y','fontweight','b');
    zlabel('Value - V','fontweight','b');
    title('Natural Interpolation Method','fontweight','b');
    hold off
%}

%%  Interpolate the strains and stress to meshgrid for all SDs
f1 = figure;
    for  i = 1:1:SD.nSD
        F = scatteredInterpolant;
        F.Points = [qt.gNodes(:,1),qt.gNodes(:,2)];
        F.Values = SD.UC(i).stress(5,:)';
        F.Method = 'natural';

        [xI, yI] = meshgrid(1:1:513);
        
        vI = F(xI,yI);
        
        %shift xI, yI according to SD position
        points = SD.gNodes(SD.nodes(i,:),:);
        xMin = min(points(:,1))
        yMin = min(points(:,2))
        xI = xI + xMin-1;
        yI = yI + yMin-1;
        
        
        hold on; 
        pcolor(xI,yI,vI);
        shading interp;
        axis equal
        axis off
        %contour(xI,yI,vI,'LineColor','k');
        %xlabel('X','fontweight','b'), ylabel('Y','fontweight','b');
        %zlabel('Value - V','fontweight','b');
        %title('Natural Interpolation Method','fontweight','b');
        hold off
    end
        
%%  Interpolate the displacements to meshgrid for all SDs
f2 = figure;
    for  i = 1:1:SD.nSD
        F = scatteredInterpolant;
        F.Points = [qt.gNodes(:,1),qt.gNodes(:,2)];
        Mag = sqrt(SD.U(SD.gDOF(i,1:2:end),:).^2+SD.U(SD.gDOF(i,2:2:end),:).^2);
        F.Values = Mag;
        F.Method = 'natural';

        [xI, yI] = meshgrid(1:1:513);
        
        vI = F(xI,yI);
        
        %shift xI, yI according to SD position
        points = SD.gNodes(SD.nodes(i,:),:);
        xMin = min(points(:,1))
        yMin = min(points(:,2))
        xI = xI + xMin-1;
        yI = yI + yMin-1;
        
        
        hold on; 
        pcolor(xI,yI,vI);
        shading interp;
        axis equal
        axis off
        %contour(xI,yI,vI,'LineColor','k');
        %xlabel('X','fontweight','b'), ylabel('Y','fontweight','b');
        %zlabel('Value - V','fontweight','b');
        %title('Natural Interpolation Method','fontweight','b');
        hold off
    end
%%  Plot displaced shape
%{
    %general parameters and values
    scaleFactor = 10;
    
    %multiscale SBFEM coarse
    x = SD.gNodes(:,1);
    y = SD.gNodes(:,2);
    
    uX_SB_f = SD.U(1:2:end);
    uY_SB_f = SD.U(2:2:end);
    
    dispX_SB_f = x + scaleFactor * uX_SB_f;
    dispY_SB_f = y + scaleFactor * uY_SB_f;
    
    %plot
    f3 = figure;
    hold on
    scatter(x,y)
    scatter(dispX_SB_f,dispY_SB_f)
    legend('nodes','SBFEM full wall')
    axis off
    axis equal
    hold off
%}   
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
    
    
   