function [ qt, MS, blocks, RVE ] = calcRVEStrains(n,qt,MS,blocks,RVE)
    %each node is given by eta = -1;
    eta = -1;
    %determine the coarse displacements of this RVE
    cDisp = RVE.U(RVE.DOF(n,:));
    %determine the fine displacements of this RVE
    uDisp = MS.NBF * cDisp;
    
    for s = 1:1:qt.nBlocks
        %determine the displacements for this block of the RVE
        bDisp = uDisp(blocks(s).gDOF);
        %determine c[] for each block
        blocks(s).c = blocks(s).Q_u \ bDisp;
        
        [ numberOfElements , ~ ] = size(blocks(s).elements);
        poissonRatio = 0.25;
        constitutiveLaw = 2; %1 = Plain strain 2 = plain stress 
        
        globalNodes = blocks(s).gDOF(2:2:end)./2;
        
        %determine strains at each node
        for i = 1:1:numberOfElements
            S_neg = diag(blocks(s).eig);
            %recall (x,y)-coordinates of each elements nodes
            xCoord = [blocks(s).elements(i,1);blocks(s).elements(i,3)] - blocks(s).sCX;
            yCoord = [blocks(s).elements(i,2);blocks(s).elements(i,4)] - blocks(s).sCY;

            %recall material properties for each element to form corresponding matrix
            [ D ] = constitutiveMatrix2D( blocks(s).EMod , poissonRatio , constitutiveLaw );

            %recall specific shape functions and their derivatives
            [shapeFunction , shapeFunctionDerivative] = shapeFunction1D (qt.nodesPerElement , eta);

            numberOfShapeFunctions = length(shapeFunction);

            %recall interpolation of cartesian coordinates (x,y) and their derivates
            x   = shapeFunction * xCoord;
            y   = shapeFunction * yCoord;
            x_n = shapeFunctionDerivative * xCoord;
            y_n = shapeFunctionDerivative * yCoord;

            %calculate specific jacobian and it's determinant
            detJ = x * y_n - y * x_n;
            %correct for negative determinant --> don't know why it is negative all of a sudden???
            detJ = -detJ;
            %specify shape function for each DOF
                %initialize shape function and it's derivatives in 2D
                shapeFunction2D           = zeros( qt.DOFPerNode , qt.DOFPerNode * qt.nodesPerElement );
                shapeFunctionDerivative2D = zeros( qt.DOFPerNode , qt.DOFPerNode * qt.nodesPerElement );

                %construct shape functions and associated derivaties in 2D
                positionCounter = 1;
                for k = 1:1:numberOfShapeFunctions
                    %abbreviation for simple notation
                    positionRow = 1:qt.DOFPerNode;
                    positionCol = positionCounter : positionCounter + qt.DOFPerNode - 1;

                    shapeFunction2D(positionRow,positionCol)           = shapeFunction(k)           * eye(2);
                    shapeFunctionDerivative2D(positionRow,positionCol) = shapeFunctionDerivative(k) * eye(2);
                    positionCounter = positionCounter + qt.DOFPerNode;
                end

            %calculate strain desplacement matrices B1 B2
            B1 =   1/detJ * [y_n ,   0 ;...
                               0 , -x_n;...
                            -x_n ,  y_n] * shapeFunction2D;
            B2 =   1/detJ * [ -y ,   0 ;...
                               0 ,   x ;...
                               x ,  -y ] * shapeFunctionDerivative2D;
                           
           %find element DOF submatrix of Q_u_SD
            pos  = blocks(s).lConn(i,:);
            Q_u_EL = blocks(s).Q_u(pos,:);    
            
            %calculate stress mode for element
            psi_sigma = D * ( -B1 * Q_u_EL * S_neg + B2 * Q_u_EL);
            
            stress = psi_sigma * blocks(s).c;
            strain = stress/blocks(s).EMod;
            
            %save stress and strain values
                %determine global node number
                RVE.UC(n).strain(1:3,globalNodes(i)) = RVE.UC(n).strain(1:3,globalNodes(i))+strain;
                RVE.UC(n).stress(1:3,globalNodes(i)) = RVE.UC(n).stress(1:3,globalNodes(i))+stress;
                RVE.UC(n).strain(4,globalNodes(i)) = RVE.UC(n).strain(4,globalNodes(i))+1;
                RVE.UC(n).stress(4,globalNodes(i)) = RVE.UC(n).stress(4,globalNodes(i))+1;
        end
    end
    %average stress and strain for RVE
    for i = 1:1:length(qt.gNodes)
        RVE.UC(n).strain(1:3,i) = real(RVE.UC(n).strain(1:3,i) ./ RVE.UC(n).strain(4,i));
        RVE.UC(n).stress(1:3,i) = real(RVE.UC(n).stress(1:3,i) ./ RVE.UC(n).stress(4,i));       
        %calculate von Mises stress
        s_x  = RVE.UC(n).stress(1,i);
        s_y  = RVE.UC(n).stress(2,i);
        s_xy = RVE.UC(n).stress(3,i);
        RVE.UC(n).stress(5,i) = real(sqrt( s_x.^2 - s_x .* s_y + s_y.^2 + 3.*s_xy.^2 ));   
    end
    
end























