function [ SIF , pSigma ] = calcSIFsRaw_3N(extension,dist,qt,block)
%%  Parameters
    poissonRatio = 0.25;
    constitutiveLaw = 2; %1 = Plain strain 2 = plain stress 
    [ numberOfElements , eleCol ] = size(block.elements);
    qt.nodesPerElement = eleCol / 2;
    %parse variables
        eEl  = extension(1,1);
        eEta = extension(1,2);

        elementDOF = block.lConn(eEl,:);

        eigenValues = block.eig;
        singularModePositions = find(eigenValues>-0.9 & eigenValues <-0.1);
        
    %Perform raw stress calculations for intersection point
       %recall (x,y)-coordinates of each elements nodes
        xCoord = [block.elements(eEl,1);block.elements(eEl,3);block.elements(eEl,5)] - block.sCX;
        yCoord = [block.elements(eEl,2);block.elements(eEl,4);block.elements(eEl,6)] - block.sCY;

        %recall material properties for each element to form corresponding matrix
        [ D ] = constitutiveMatrix2D( block.EMod , poissonRatio , constitutiveLaw );

        %recall specific shape functions and their derivatives
        [shapeFunction , shapeFunctionDerivative] = shapeFunction1D (qt.nodesPerElement , eEta);

        numberOfShapeFunctions = length(shapeFunction);

        %recall interpolation of cartesian coordinates (x,y) and their derivates
        x   = shapeFunction * xCoord;
        y   = shapeFunction * yCoord;
        x_n = shapeFunctionDerivative * xCoord;
        y_n = shapeFunctionDerivative * yCoord;

        %calculate specific jacobian and it's determinant
        detJ = x * y_n - y * x_n;
        %correct for negative determinant --> don't know why it is negative all of a sudden???
        detJ = -detJ;
        
        %specify shape function for each DOF
            %initialize shape function and it's derivatives in 2D
                shapeFunction2D           = zeros( qt.DOFPerNode , qt.DOFPerNode * qt.nodesPerElement );
                shapeFunctionDerivative2D = zeros( qt.DOFPerNode , qt.DOFPerNode * qt.nodesPerElement );

            %construct shape functions and associated derivaties in 2D
                positionCounter = 1;
                for k = 1:1:numberOfShapeFunctions
                    %abbreviation for simple notation
                    positionRow = 1:qt.DOFPerNode;
                    positionCol = positionCounter : positionCounter + qt.DOFPerNode - 1;

                    shapeFunction2D(positionRow,positionCol)           = shapeFunction(k)           * eye(2);
                    shapeFunctionDerivative2D(positionRow,positionCol) = shapeFunctionDerivative(k) * eye(2);
                    positionCounter = positionCounter + qt.DOFPerNode;
                end

            %calculate strain desplacement matrices B1 B2
            B1 =   1/detJ * [y_n ,   0 ;...
                               0 , -x_n;...
                            -x_n ,  y_n] * shapeFunction2D;
            B2 =   1/detJ * [ -y ,   0 ;...
                               0 ,   x ;...
                               x ,  -y ] * shapeFunctionDerivative2D;

        %extract values to calculate SIFs based on raw stresses
            %singular eigenvalues
            S1 = eigenValues(singularModePositions(1,1),1);
            S2 = eigenValues(singularModePositions(2,1),1);
            %contributions of boundary conditions
            c1 = block.c(singularModePositions(1,1), 1);
            c2 = block.c(singularModePositions(2,1), 1);
            %mode for element
            psi = block.Q_u(elementDOF,:);
            %raw stress mode
            psiSigma1 = D * (-B1 * psi * S1 + B2 * psi);
            psiSigma2 = D * (-B1 * psi * S2 + B2 * psi);
            pSigma = [psiSigma1(:,singularModePositions(1,1));psiSigma2(:,singularModePositions(2,1))];
            %raw stresses
            sigma = ( psiSigma1( : , singularModePositions( 1,1 ) ) * c1 + psiSigma2( : , singularModePositions( 2,1 ) ) * c2 );
        
            K_raw = sqrt(2 * pi * dist) * sigma;
            SIF = real(K_raw);
            %}
end





















