function [F] = getForceVector(block, qt)
    %parameters for numerical integration of forces
    F = zeros(qt.numberOfDOF,1);
    DOFPerElement = qt.nodesPerElement * qt.DOFPerNode;
    
    %umerical integration values
    [ samplingPoints , weights ] = integrationParameters1D ( qt.nodesPerElement , qt.quadratureType);
    numberOfSamplingPoints = length(samplingPoints);
    %for the elements of the second side
    for i = 1:1:length(qt.topBlocks)
        %initialize loading vector
        F_t = zeros(DOFPerElement,1);
        F_s = zeros(DOFPerElement,1);
        %determine element properties
        xCoord = block(qt.topBlocks(i)).nodes(end-1:end,1);
        yCoord = block(qt.topBlocks(i)).nodes(end-1:end,2);
        elementLength = sqrt((xCoord(1,:)-xCoord(end,:))^2 + (yCoord(1,:)-yCoord(end,:))^2);
        detJ = elementLength/2;
        
        %proceed with numerical integration
        for j = 1:1:numberOfSamplingPoints
            
            samplingPoint = samplingPoints(j);
            [shapeFunction , ~] = shapeFunction1D (qt.nodesPerElement , samplingPoint);

            numberOfShapeFunctions = length(shapeFunction);

            %recall specific integration weights
            weight = weights(j);
            
            %specify shape function for each DOF
            %initialize shape function and it's derivatives in 2D
                shapeFunction2D           = zeros( qt.DOFPerNode , qt.DOFPerNode * qt.nodesPerElement );
            %construct shape functions and associated derivaties in 2D
                positionCounter = 1;
                for k = 1:1:numberOfShapeFunctions
                    %abbreviation for simple notation
                    positionRow = 1:qt.DOFPerNode;
                    positionCol = positionCounter : positionCounter + qt.DOFPerNode - 1;

                    shapeFunction2D(positionRow,positionCol)           = shapeFunction(k) * eye(2);
                    positionCounter = positionCounter + qt.DOFPerNode;
                end
           F_t = F_t + shapeFunction2D' * [0,qt.load]' .* detJ .* weight; %tension
           F_s = F_s + shapeFunction2D' * [-qt.load,0]' .* detJ .* weight; %shear
        end
        
        %retrieve DOFs of each element
        ind = [block(qt.topBlocks(i)).gDOF(end-1:end),block(qt.topBlocks(i)).gDOF(1:2)];
        %assemble local force vector into global one   
        F(ind,1) = F(ind,1) + F_t + F_s;
     
    end
end