



































function [ MS ] = getMsNBF(MS, qt)
    %general parameters
    nSides = 4;
    nodesPerSide = MS.nCNodes/nSides+1;
    MS.cNodes = [];
    %generate test positions for coarse Nodes
    for i = 1:1:nSides
        %test a linear spacing and then find the closest corresponding node
        switch i
            case 1
                testNodes = linspace(1,qt.imageSize+1,nodesPerSide);
                MS.cNodes = [MS.cNodes;[ones(length(testNodes(1:end-1)),1),testNodes(1:end-1)']];
            case 2
                testNodes = linspace(1,qt.imageSize+1,nodesPerSide);
                MS.cNodes = [MS.cNodes;[testNodes(1:end-1)',ones(length(testNodes(1:end-1)),1).*(qt.imageSize+1)]];
            case 3
                testNodes = linspace(qt.imageSize+1,1,nodesPerSide);
                MS.cNodes = [MS.cNodes;[ones(length(testNodes(1:end-1)),1).*(qt.imageSize+1),testNodes(1:end-1)']];
            case 4
                testNodes = linspace(qt.imageSize+1,1,nodesPerSide);
                MS.cNodes = [MS.cNodes;[testNodes(1:end-1)',ones(length(testNodes(1:end-1)),1)]];
            otherwise
        end
    end
       
    %generate matrix of all boundary nodes
        MS.BCNodes = [];
        for i = [1,4,3,2]
            switch i
                case {1,4}
                    MS.BCNodes = [MS.BCNodes;[MS.BC(i).nodes,MS.BC(i).row]];
                case {2,3}
                    MS.BCNodes = [MS.BCNodes;flipud([MS.BC(i).nodes,MS.BC(i).row])];
            end
            
        end
        MS.BCNodes = unique(MS.BCNodes,'rows','stable');
        MS.nBCNodes = length(MS.BCNodes);
        %generate DOFs for all the boundary Nodes
        MS.BCDOF = repmat(MS.BCNodes(:,3),[1,2]);
        MS.BCDOF(:,1) = MS.BCDOF(:,1)*qt.DOFPerNode-1;
        MS.BCDOF(:,2) = MS.BCDOF(:,2)*qt.DOFPerNode;
        MS.BCDOF = MS.BCDOF';
        MS.BCDOF = reshape(MS.BCDOF,1,[]);
        
    %find the nearest existing node for each of MS.cNodes current testing nodes    
        minimum = [];
        index = [];
        for i = 1:1:MS.nCNodes
            tMin = inf;
            cNode = MS.cNodes(i,:);
            for j = 1:1:MS.nBCNodes
                distance = sqrt((cNode(1,1)-MS.BCNodes(j,1))^2+(cNode(1,2)-MS.BCNodes(j,2))^2);
                if distance < tMin
                    tMin = distance;
                    tInd = j;
                end
            end
            minimum = [minimum,tMin];
            index = [index, tInd];
        end
        MS.cNodes = MS.BCNodes(index,1:2);   %node positions
        MS.cRows  = MS.BCNodes(index,3);     %gNode indices
        
    %determine how boundary nodes are linked to coarse nodes
        [~,MS.cIndex]=ismember(MS.cNodes,MS.BCNodes(:,1:2),'rows');
    
    %calculate NBF
        %initialize
        MS.NBF = [];  
        
        switch MS.BCType
            case 1  %linear
                DOF_fixed = MS.BCDOF;
                DOF_free = setdiff(1:1:qt.numberOfDOF, DOF_fixed);
                for i = 1:1:MS.nCNodes * qt.DOFPerNode
                    %disp(i)
                    %initialize
                    DOFs = zeros(1,qt.numberOfDOF);
                    F_system = zeros(qt.numberOfDOF,1);
                    K_system = qt.kFine;
                    %determine which coarse node
                    coarseNode = ceil(i/qt.DOFPerNode);
                    %determine involved boundary nodes
                    if coarseNode == 1  %wrap around for first ramp
                        sNode = [MS.cNodes(end,:),MS.cRows(end,:),MS.cIndex(end,:)];
                        iNode = [MS.cNodes(coarseNode  ,:),MS.cRows(coarseNode  ,:),MS.cIndex(coarseNode  ,:)];
                        eNode = [MS.cNodes(coarseNode+1,:),MS.cRows(coarseNode+1,:),MS.cIndex(coarseNode+1,:)];
                        amp1Nodes = [sNode(4):MS.nBCNodes,1];
                        amp2Nodes = [iNode(4):eNode(4)];
                    elseif coarseNode == MS.nCNodes     %wrap around for last ramp
                        sNode = [MS.cNodes(coarseNode-1,:),MS.cRows(coarseNode-1,:),MS.cIndex(coarseNode-1,:)];
                        iNode = [MS.cNodes(coarseNode  ,:),MS.cRows(coarseNode  ,:),MS.cIndex(coarseNode  ,:)];
                        eNode = [MS.cNodes(1,:),MS.cRows(1,:),MS.cIndex(1,:)];
                        amp1Nodes = [sNode(4):iNode(4)];
                        amp2Nodes = [iNode(4):MS.nBCNodes,1];
                    else
                        sNode = [MS.cNodes(coarseNode-1,:),MS.cRows(coarseNode-1,:),MS.cIndex(coarseNode-1,:)];
                        iNode = [MS.cNodes(coarseNode  ,:),MS.cRows(coarseNode  ,:),MS.cIndex(coarseNode  ,:)];
                        eNode = [MS.cNodes(coarseNode+1,:),MS.cRows(coarseNode+1,:),MS.cIndex(coarseNode+1,:)];
                        amp1Nodes = [sNode(4):iNode(4)];
                        amp2Nodes = [iNode(4):eNode(4)];
                    end
                    %determine the amplitude of each involved boundary node
                    length1 = sqrt((sNode(1)-iNode(1))^2+(sNode(2)-iNode(2))^2);
                    length2 = sqrt((iNode(1)-eNode(1))^2+(iNode(2)-eNode(2))^2);
                    
                    amp1Value = zeros(1,length(amp1Nodes));
                    amp2Value = zeros(1,length(amp2Nodes));
                    
                    for j = 1:1:length(amp1Nodes)
                        temp1 = MS.BCNodes(amp1Nodes(j),1:2);
                        tDist1 = sqrt((sNode(1)-temp1(1))^2+(sNode(2)-temp1(2))^2);
                        amp1Value(1,j) = tDist1 / length1;
                    end
                    for j = 1:1:length(amp2Nodes)
                        temp2 = MS.BCNodes(amp2Nodes(j),1:2);
                        tDist2 = sqrt((eNode(1)-temp2(1))^2+(eNode(2)-temp2(2))^2);
                        amp2Value(1,j) = tDist2 / length2;
                    end

                    if mod(i,2) == 1 %unit displacement in x-direction
                        amp1DOF = MS.BCNodes(amp1Nodes,3)*2-1;
                        amp2DOF = MS.BCNodes(amp2Nodes,3)*2-1;
                    else %unit displacement in y-direction
                        amp1DOF = MS.BCNodes(amp1Nodes,3)*2;
                        amp2DOF = MS.BCNodes(amp2Nodes,3)*2;
                    end
                    
                    DOFs(amp1DOF) = amp1Value;
                    DOFs(amp2DOF) = amp2Value;
                    
                    %find displacements of each DOF 
                    Disp_fixed = DOFs(DOF_fixed);
                    %eqivalent force vector due to prescribed displacements
                    F_system(DOF_free ,1) = -K_system(DOF_free,DOF_fixed)*Disp_fixed';
                    F_system(DOF_fixed,1) = Disp_fixed';
                    %alterations to stiffness matrix
                    nFixed = length(DOF_fixed);
                    nFree  = qt.numberOfDOF - nFixed;
                    K_system(DOF_free ,DOF_fixed) = zeros(nFree,nFixed);
                    K_system(DOF_fixed,DOF_free)  = zeros(nFixed,nFree);
                    K_system(DOF_fixed,DOF_fixed) = eye(nFixed,nFixed);
                    %solve equilibrium equation
                    u_system = K_system \ F_system; %<-- cursory inspection ok, all disp <= 1
                    %update G-matrix
                    MS.NBF = [MS.NBF,u_system];
                end
            case 2  %periodic
                %turn off warnings
                warning('off','all')
                %parameters
                nLDOF = length(MS.BC(1).DOF);
                nTDOF = length(MS.BC(2).DOF);
                prescribed = 1;
                restrained = qt.DOFPerNode * MS.nCNodes - 1;
                rows  = nLDOF + nTDOF + prescribed + restrained;
                %initialize
                K_system = qt.kFine;
                F_system = zeros(qt.numberOfDOF,1);
                MS.A = zeros(rows, qt.numberOfDOF);
                %setup A_system
                for  i = 1:1:nLDOF
                    MS.A(i,MS.BC(1).DOF(i)) =  1;
                    MS.A(i,MS.BC(3).DOF(i)) = -1;
                end
                for  i = 1:1:nTDOF
                    MS.A(i+nLDOF,MS.BC(2).DOF(i)) =  1;
                    MS.A(i+nLDOF,MS.BC(4).DOF(i)) = -1;
                end
                %generate DOFs for all coarse Node DOFs
                    MS.cDOF = repmat(MS.cRows,[1,2]);
                    MS.cDOF(:,1) = MS.cDOF(:,1)*qt.DOFPerNode-1;
                    MS.cDOF(:,2) = MS.cDOF(:,2)*qt.DOFPerNode;
                    MS.cDOF = MS.cDOF';
                    MS.cDOF = reshape(MS.cDOF,1,[]);
                %assign corresponding augmentations to matrix
                for i = 1:1:length(MS.cDOF)
                    MS.A(i+nLDOF+nTDOF,MS.cDOF(i)) = 1;
                end
                
                %NBF specific operations
                for i = 1:1:MS.nCNodes * qt.DOFPerNode
                    %prediscribed DOF in B
                    MS.B = zeros(rows,1);
                    MS.B(i+nLDOF+nTDOF,1) = 1;
                    
                    %ramp of multi point constraint values for B
                        %determine which coarse node
                        coarseNode = ceil(i/qt.DOFPerNode);
                        %determine involved boundary nodes
                        if coarseNode == 1  %wrap around for first ramp
                            sNode = [MS.cNodes(end,:),MS.cRows(end,:),MS.cIndex(end,:)];
                            iNode = [MS.cNodes(coarseNode  ,:),MS.cRows(coarseNode  ,:),MS.cIndex(coarseNode  ,:)];
                            eNode = [MS.cNodes(coarseNode+1,:),MS.cRows(coarseNode+1,:),MS.cIndex(coarseNode+1,:)];
                            amp1Nodes = [sNode(4):MS.nBCNodes,1];
                            amp2Nodes = [iNode(4):eNode(4)];
                        elseif coarseNode == MS.nCNodes     %wrap around for last ramp
                            sNode = [MS.cNodes(coarseNode-1,:),MS.cRows(coarseNode-1,:),MS.cIndex(coarseNode-1,:)];
                            iNode = [MS.cNodes(coarseNode  ,:),MS.cRows(coarseNode  ,:),MS.cIndex(coarseNode  ,:)];
                            eNode = [MS.cNodes(1,:),MS.cRows(1,:),MS.cIndex(1,:)];
                            amp1Nodes = [sNode(4):iNode(4)];
                            amp2Nodes = [iNode(4):MS.nBCNodes,1];
                        else
                            sNode = [MS.cNodes(coarseNode-1,:),MS.cRows(coarseNode-1,:),MS.cIndex(coarseNode-1,:)];
                            iNode = [MS.cNodes(coarseNode  ,:),MS.cRows(coarseNode  ,:),MS.cIndex(coarseNode  ,:)];
                            eNode = [MS.cNodes(coarseNode+1,:),MS.cRows(coarseNode+1,:),MS.cIndex(coarseNode+1,:)];
                            amp1Nodes = [sNode(4):iNode(4)];
                            amp2Nodes = [iNode(4):eNode(4)];
                        end
                        %determine the amplitude of each involved boundary node
                        length1 = sqrt((sNode(1)-iNode(1))^2+(sNode(2)-iNode(2))^2);
                        length2 = sqrt((iNode(1)-eNode(1))^2+(iNode(2)-eNode(2))^2);

                        amp1Value = zeros(1,length(amp1Nodes));
                        amp2Value = zeros(1,length(amp2Nodes));

                        for j = 1:1:length(amp1Nodes)
                            temp1 = MS.BCNodes(amp1Nodes(j),1:2);
                            tDist1 = sqrt((sNode(1)-temp1(1))^2+(sNode(2)-temp1(2))^2);
                            amp1Value(1,j) = tDist1 / length1;
                        end
                        for j = 1:1:length(amp2Nodes)
                            temp2 = MS.BCNodes(amp2Nodes(j),1:2);
                            tDist2 = sqrt((eNode(1)-temp2(1))^2+(eNode(2)-temp2(2))^2);
                            amp2Value(1,j) = tDist2 / length2;
                        end

                        %determine corresponding ampDOF
                        if mod(i,2) == 1 %unit displacement in x-direction
                            amp1DOF = MS.BCNodes(amp1Nodes,3)*2-1;
                            amp2DOF = MS.BCNodes(amp2Nodes,3)*2-1;
                        else %unit displacement in y-direction
                            amp1DOF = MS.BCNodes(amp1Nodes,3)*2;
                            amp2DOF = MS.BCNodes(amp2Nodes,3)*2;
                        end
                    
                        %add ampValues to corresponding ampDOF
                        for j = 1:1:length(amp1DOF)
                            temp = MS.A(1:nLDOF+nTDOF,amp1DOF(j));
                            [row,~,val] = find(temp);
                            MS.B(row,1) = amp1Value(1,j).*sign(val);
                        end
                        for j = 1:1:length(amp2DOF)
                            temp = MS.A(1:nLDOF+nTDOF,amp2DOF(j));
                            [row,~,val] = find(temp);
                            MS.B(row,1) = amp2Value(1,j).*sign(val);
                        end
                        
                        if MS.nCNodes == 4
                            switch i
                                case 1;MS.B(155,1) = 1; 
                                case 2;MS.B(156,1) = 1; 
                                case 3;MS.B(153,1) = 1;
                                case 4;MS.B(154,1) = 1;
                                case 5;MS.B(411,1) = -1;
                                case 6;MS.B(412,1) = -1;
                                case 7;MS.B(1,1) = -1;
                                case 8;MS.B(2,1) = -1;
                            end
                        end

                        U_system = [K_system,MS.A';MS.A,zeros(rows,rows)]\[F_system;MS.B];
                        U_system = U_system(1:qt.numberOfDOF,1);
                        MS.NBF = [MS.NBF,U_system];
                end
                %turn on warning again
                warning('on','all')
                
        end 
    
end

































