%%  General procedure for MSBFEM
%1.   X = (x,y)-coordinates of result outputs --> here nodes
%2.   V = result output at X-point
%3.   F = object with properties:
%       F.points --> X
%       F.Values --> V
%       F.Method --> interpolation method (inside domain)
%       F.ExtrapolationMethod --> extrapolation (outside of domain)
%4.   [xi,yi] = meshgrid to create plotting mesh points
%5.   vi = F(xi,yi) to interpolate to scattered points at (xi,yi) using F.Method
%6.   surf(xi,yi,vi) to plot data interpolated to meshgrid


%%  Displacements

    F = scatteredInterpolant;
    F.Points = [qt.gNodes(:,1),qt.gNodes(:,2)];
    F.Values = qt.uFine(1:2:end); %x-displacements
    %F.Values = qt.uFine(2:2:end); %y-displacements
    F.Method = 'natural';
    
    [xI, yI] = meshgrid(1:1:513);
    
    vI = F(xI,yI);
    
    %F.Values = qt.uFine(2:2:end); %y-displacements
    %vIY = F(xI,yI);
    
    %surf(xI,yI,vI);
    %contourf(xI,yI,vI);
    %quiver(xI,yI,vIX,vIY)
    
   
    hold on; 
    pcolor(vI);
    colormap(parula(1000))
    shading interp;
    axis equal
    contour(xI,yI,vI,'LineColor','k');
    xlabel('X','fontweight','b'), ylabel('Y','fontweight','b');
    zlabel('Value - V','fontweight','b');
    title('Natural Interpolation Method','fontweight','b');
    hold off
    

%%  Matlab example
%{
X = -3 + 6.*gallery('uniformdata',[250 2],0);
V = peaks(X(:,1),X(:,2));

F = scatteredInterpolant(X,V)

[Xq,Yq] = meshgrid(-2.5:0.125:2.5);
Vq = F(Xq,Yq);
surf(Xq,Yq,Vq);
xlabel('X','fontweight','b'), ylabel('Y','fontweight','b');
zlabel('Value - V','fontweight','b');
title('Linear Interpolation Method','fontweight','b');

F.Method = 'natural';
Vq = F(Xq,Yq);
figure
surf(Xq,Yq,Vq);
xlabel('X','fontweight','b'),ylabel('Y','fontweight','b') 
zlabel('Value - V','fontweight','b')
title('Natural neighbor Interpolation Method','fontweight','b');
%}