%%  qtMSBFEM
    %tic
%% Build RVE

    qt.imageSize = 512;
    qt.image = zeros(qt.imageSize);

    %define all mortar and brick portions of RVE (as detailed in IPE masonry.pdf)
    qt.Color.Mortar = 0.8;
    qt.Color.Brick  = 0.6;
    
    %assign all brick and then overwrite portions with mortar
    qt.image(:,:) = qt.Color.Brick;
    %horizontal mortar
    qt.image(1:4,:)       = qt.Color.Mortar;
    qt.image(252:260,:)   = qt.Color.Mortar;
    qt.image(end-3:end,:) = qt.Color.Mortar;
    %vertical mortar
    qt.image(1:260,1:4)       = qt.Color.Mortar;
    qt.image(1:260,252:260)   = qt.Color.Mortar;
    qt.image(1:260,end-3:end) = qt.Color.Mortar;
    qt.image(260:end,124:132) = qt.Color.Mortar;
    qt.image(260:end,380:388) = qt.Color.Mortar;
    
    %imshow(qt.image)
    
%%  Perform quadtree decomposition
    %perfrom decomposition
    qt.decompMin =  4;
    qt.decompMax = 64;
    qt.decomp = qtdecomp(qt.image,0.2,[qt.decompMin,qt.decompMax]);
    %display current blocks
    %{
        block = repmat(uint8(0),size(qt.decomp));
        for dim = [512 256 128 64 32 16 8 4 2 1]   
          numblocks = length(find(qt.decomp==dim));    
          if (numblocks > 0)        
            values = repmat(uint8(1),[dim dim numblocks]);
            values(2:dim,2:dim,:) = 0;
            block = qtsetblk(block,qt.decomp,dim,values);
          end
        end
        block(end,1:end) = 1;
        block(1:end,end) = 1;
        %imshow(qt.image)
        figure
        imshow(block,[])
    %}
    %find size and positions of quadtree blocks
    [qt.rows, qt.cols, qt.size] = find(qt.decomp);
    %find amount of blocks
    qt.nBlocks = length(qt.rows);
    %find values in each block
    qt.values = zeros(qt.nBlocks,1);
    for  i = 1:1:qt.nBlocks
        %find color values of each block
        qt.values(i) = qt.image(qt.rows(i),qt.cols(i));
        %replicate block size values on border
        r = qt.rows(i);
        c = qt.cols(i);
        qt.decomp(r,c:c+qt.size(i)-1)              = qt.size(i); %upper row
        qt.decomp(r:r+qt.size(i)-1,c)              = qt.size(i); %left column
        qt.decomp(r+qt.size(i)-1,c:c+qt.size(i)-1) = qt.size(i); %bottom row
        qt.decomp(r:r+qt.size(i)-1,c+qt.size(i)-1) = qt.size(i); %right column
    end
    
    %spy(qt.decomp)
    
%%  Define global parameters
    qt.tol = 10^-3;
    qt.DOFPerNode = 2;
    qt.nodesPerElement = 2;
    qt.quadratureType = 1; %Gauss
    qt.load = -0.1;
    qt.EMod.Brick  = 11;%11.0/3;
    qt.EMod.Mortar =  7;% 7.2/30;

%%  Generate information on each block (old subdomains)
    %initialize array of block structures (performance considerations)
        blocks(qt.nBlocks).nodes = [];
        blocks(qt.nBlocks).nNodes = [];
        blocks(qt.nBlocks).eNode = [];
        blocks(qt.nBlocks).sNode = [];
        blocks(qt.nBlocks).elements = [];
        blocks(qt.nBlocks).EMod = [];
        blocks(qt.nBlocks).sCX = [];
        blocks(qt.nBlocks).sCY = [];
        blocks(qt.nBlocks).size = [];
        blocks(qt.nBlocks).row = [];
        blocks(qt.nBlocks).col = [];
        blocks(qt.nBlocks).gDOF = [];
        blocks(qt.nBlocks).lConn = [];
        gNodes = [];
        
        nodesPerElement = 2;
        
    for i = 1:1:qt.nBlocks
        %find boundary discretization counter clockwise from north-west corner
            %--> leads to elements matrix with discrete (x,y)-values
        %recall block size and position
            blocks(i).size  = qt.size(i);
            blocks(i).row   = qt.rows(i);
            blocks(i).col   = qt.cols(i);
            blocks(i).value = qt.values(i);
        
        %iterate over each side of square domain
        lNodes = [];  
        nSides = 4;            
        nElement = zeros(1,nSides);
        for j = 1:1:nSides
            counter = 0;
            nodePerSide = 0;

            switch j
                case 1
                    %generate nodes
                    while counter <= blocks(i).size
                        lNodes = [lNodes;[blocks(i).row+counter,blocks(i).col]];
                        nodePerSide = nodePerSide + 1;
                        
                        row = min(blocks(i).row+counter,qt.imageSize);
                        col = max(1,blocks(i).col-1);             
                        val = min(qt.decomp(row,col),blocks(i).size);

                        counter = counter + val;   
                    end
                    nElement(1,j) =  nodePerSide-1;
                    
                case 2
                    %generate nodes
                    while counter <= blocks(i).size
                        lNodes = [lNodes;[blocks(i).row+blocks(i).size,blocks(i).col+counter]];
                        nodePerSide = nodePerSide + 1;
                        
                        row = min(blocks(i).row+blocks(i).size,qt.imageSize);
                        col = min(blocks(i).col+counter,qt.imageSize);
                        val = min(qt.decomp(row,col),blocks(i).size);

                        counter = counter + val; 
                    end
                    nElement(1,j) =  nodePerSide - 1;
                    
                case 3
                    %generate nodes
                    while counter <= blocks(i).size
                        lNodes = [lNodes;[blocks(i).row+counter,blocks(i).col+blocks(i).size]];
                        nodePerSide = nodePerSide + 1;
                        
                        row = min(blocks(i).row+counter,qt.imageSize);
                        col = min(blocks(i).col+blocks(i).size,qt.imageSize);
                        val = min(qt.decomp(row,col),blocks(i).size);

                        counter = counter + val;
                    end
                    nElement(1,j) =  nodePerSide - 1;
                    
                case 4
                    %generate nodes
                    while counter <= blocks(i).size
                        lNodes = [lNodes;[blocks(i).row,blocks(i).col+counter]];
                        nodePerSide = nodePerSide + 1;
                        
                        row = max(blocks(i).row-1,1);
                        col = min(blocks(i).col+counter,qt.imageSize);
                        val = min(qt.decomp(row,col),blocks(i).size);

                        counter = counter + val;
                    end
                    nElement(1,j) =  nodePerSide - 1;
                    
                otherwise
                disp('switch error pos/val/lNodes')    
            end 
        end

        %swap columns to correct (x,y)-coordinates
        lNodes(:,[1,2]) = lNodes(:,[2,1]);
        %correct element numbering orientation to counter clockwise
        tS = sum(nElement(1:2))+length(nElement(1:2))+1;
        tE = tS + nElement(3);
        lNodes(tS:tE,:) = flipud(lNodes(tS:tE,:));
        
        tS = sum(nElement(1:3))+length(nElement(1:3))+1;
        tE = tS + nElement(4);
        lNodes(tS:tE,:) = flipud(lNodes(tS:tE,:));
        %correct double nodes
        temp = lNodes(1,:);
        lNodes = unique(lNodes,'rows','stable');
        blocks(i).nodes = [lNodes;temp];
        blocks(i).nNodes = length(blocks(i).nodes);
        blocks(i).eNode = sum([blocks(1:i).nNodes]);
        blocks(i).sNode = blocks(i).eNode-blocks(i).nNodes+1;
           
        %create elements for each block
        for j = 1:1:length(blocks(i).nodes)-1
            blocks(i).elements(j,:) = [blocks(i).nodes(j,:),blocks(i).nodes(j+1,:)];
        end

        %find center of block == scaling center
        blocks(i).sCX = mean([min(blocks(i).nodes(:,1)),max(blocks(i).nodes(:,1))]);
        blocks(i).sCY = mean([min(blocks(i).nodes(:,2)),max(blocks(i).nodes(:,2))]);
        
        %find E-mod of block
        if abs(blocks(i).value - qt.Color.Mortar) < qt.tol
            blocks(i).EMod = qt.EMod.Mortar;
        else
            blocks(i).EMod = qt.EMod.Brick;
        end
        
        %add local nodes to global nodes
        gNodes = [gNodes;blocks(i).nodes(:,:)];
    end
    
%%  plot generated blocks i.e. SBFEM domains
    %%{
    hold on
    for i = 1:1:qt.nBlocks
        %plot(blocks(i).nodes(:,1),-blocks(i).nodes(:,2))
        fill(blocks(i).nodes(:,1),-blocks(i).nodes(:,2),blocks(i).value)
    end
    axis equal
    hold off
    %}
    %{
    f2 = figure;
    scatter(gNodes(:,1),-gNodes(:,2))
    %}

%%  determine global node numbering of each block
    %perform unique (with resorting indices!) on global nodes matrix
    [qt.gNodes, indOld, indNew] = unique(gNodes,'rows','stable');
    qt.numberOfDOF = length(qt.gNodes)*qt.DOFPerNode;
    
    %initialize sparse global stiffness matrix
    qt.kFine = zeros(qt.numberOfDOF,qt.numberOfDOF);

    %generate template for local connectivety
        maximum = 0;
        for i = 1:1:qt.nBlocks
            if length(blocks(i).elements) > maximum
                maximum = length(blocks(i).elements);
                maxI = i;
            end
        end

        connectivity = zeros(maximum + 1,nodesPerElement*qt.DOFPerNode);
        connectivity(1,:) = [1,2,3,4];
        for i = 2:1:maximum + 1
            connectivity(i,:) = connectivity(1,:)+(i-1)*qt.DOFPerNode;
        end
    
    for i = 1:1:qt.nBlocks
        %assign each block its global DOF numbers based on unique sorting indices
        blockLNodes = blocks(i).sNode:1:blocks(i).eNode;
        blockGNodes = indNew(blockLNodes);
        blockGDOF = repmat(blockGNodes,[1,2]);
        blockGDOF(:,1) = blockGDOF(:,1)*qt.DOFPerNode-1;
        blockGDOF(:,2) = blockGDOF(:,2)*qt.DOFPerNode;
        blockGDOF = blockGDOF';
        blockGDOF = reshape(blockGDOF,1,[]);
        blocks(i).gDOF = blockGDOF(1:end-qt.DOFPerNode);
        
        %assign each block a local connectivety based on counter clockwise direction
        blockConnectivity = connectivity(1:blocks(i).nNodes-1,:);
        blockConnectivity(end,[3,4]) = [1,2];
        blocks(i).lConn = blockConnectivity;

        %create stiffness matrix for each block
        [blocks(i).K, blocks(i).Q_u, blocks(i).eig] = getStiffnessMatrix(blocks(i),qt);
        
        %assemble individual block stiffness matrices to global stiffness matrix
        qt.kFine(blocks(i).gDOF,blocks(i).gDOF) = qt.kFine(blocks(i).gDOF,blocks(i).gDOF) + blocks(i).K;    
    end
    qt.kFine = sparse(qt.kFine); 
    %spy(K);

% %%  Determine neighbour node pairs automatically
%     %find boundary nodes (by looking at gNodes)
%         threshold.min = 2;
%         threshold.max = qt.imageSize - threshold.min;
%         
%     %generate node position matrix, corresponding global node row-identifier and correponding DOF for each side
%         MS = [];
%         for i = 1:1:nSides
%             [ MS ] = getMsBCs(qt, threshold, i, MS);
%         end
%     
% %%  Generate numerical basis functions
%         MS.nCNodes = 8;  %must be divisible by 4 
%         MS.BCType  = 1;  %1=linear, 2=periodic
%                          %3=hybrid, 4=oversampling linear, 5=oversampling periodic  || these are not implemented yet!
%         tic;
%         [ MS ] = getMsNBF(MS, qt);    %works for linear
%         toc;
% %%  Construct coarse stiffness matrix of RVE
%     qt.kCoarse = MS.NBF'*qt.kFine*MS.NBF;
%     %create alias so that subsequent computations keep unmodified stiffness matrix
%     kCoarse = qt.kCoarse;
%  
% %%  Construct coarse force vector
%     [ MS ] = getMsForceVector( MS, qt, threshold);
%     
% %%  Define restrained nodes
%     [ MS ] = getMsRestrainedDOF( MS, qt, threshold);
%     
% %%  Apply boundary conditions 
%         %apply boundary conditions
%         kCoarse(MS.BCDOF,:) = zeros;
%         kCoarse(:,MS.BCDOF) = zeros;
%         kCoarse(MS.BCDOF,MS.BCDOF) = eye(length(MS.BCDOF));
%         
%         MS.F(MS.BCDOF,1) = zeros;
% 
% %%  Obtain course displacements
%         MS.uCoarse = kCoarse\MS.F;
%         
% %%  Reconstruct fine mesh displacements
%         MS.uFine = MS.NBF*MS.uCoarse; 
%     
%%  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Standard SBFEM solution from here on in below! 
%
%%  assemble loading vector
    %find block numbers at top of domain
    qt.topBlocks = [];
    for i = 1:1:qt.nBlocks
        if find(blocks(i).nodes(:,2) < 2)
            qt.topBlocks = [qt.topBlocks,i];
        end
    end
    
    [F] = getForceVector(blocks, qt);
    
%%  apply boundary conditions
    %find DOF at bottom of domain
    [qt.bottomDOF,~] = find(qt.gNodes(:,2) > qt.imageSize-2);
    qt.bottomDOF = repmat(qt.bottomDOF,[1,2]);
    qt.bottomDOF(:,1) = qt.bottomDOF(:,1)*qt.DOFPerNode-1;
    qt.bottomDOF(:,2) = qt.bottomDOF(:,2)*qt.DOFPerNode;
    qt.bottomDOF = qt.bottomDOF';
    qt.bottomDOF = reshape(qt.bottomDOF,1,[]);
    
    K = qt.kFine;
    K(           :,qt.bottomDOF) = zeros;
    K(qt.bottomDOF,           :) = zeros;
    K(qt.bottomDOF,qt.bottomDOF) = eye(length(qt.bottomDOF));
    
    F(qt.bottomDOF,           1) = zeros;

%%  find displacements
    qt.uFine = K\F; 
    
%%  Plot displaced shape
%{
    %general parameters and values
    scaleFactor = 10;
    
    x_f = qt.gNodes(:,1);
    y_f = qt.gNodes(:,2);
    
    %standard SBFEM fine
    uX_S = qt.uFine(1:2:end);
    uY_S = qt.uFine(2:2:end);
    
    dispX_S = x_f + scaleFactor * uX_S;
    dispY_S = y_f + scaleFactor * uY_S;
    
    %multiscale SBFEM fine
    uX_MS_f = MS.uFine(1:2:end);
    uY_MS_f = MS.uFine(2:2:end);
    
    dispX_MS_f = x_f + scaleFactor * uX_MS_f;
    dispY_MS_f = y_f + scaleFactor * uY_MS_f;
    
    %multiscale SBFEM coarse
    x_c = MS.cNodes(:,1);
    y_c = MS.cNodes(:,2);
    
    uX_MS_c = MS.uCoarse(1:2:end);
    uY_MS_c = MS.uCoarse(2:2:end);
    
    dispX_MS_c = x_c + scaleFactor * uX_MS_c;
    dispY_MS_c = y_c + scaleFactor * uY_MS_c;
    
    %plot
    f3 = figure;
    hold on
    scatter(x_f,y_f)
    scatter(dispX_S,dispY_S)
    scatter(dispX_MS_f,dispY_MS_f)
    scatter(dispX_MS_c,dispY_MS_c)
    legend('nodes','SBFEM','multiscale fine','multiscale coarse')
    axis off
    axis equal
    hold off
%}   
 
%toc;
    